package com.pms.ocp.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.pms.ocp.mapper.ModelAcceptMapper;
import com.pms.ocp.mapper.ModelAuditMapper;
import com.pms.ocp.mapper.ModelCheckMapper;
import com.pms.ocp.mapper.ModelMapper;
import com.pms.ocp.model.entity.Model;
import com.pms.ocp.model.entity.ModelAudit;
import com.pms.ocp.model.entity.ModelPropInfo;
import com.pms.ocp.model.entity.PropInfo;
import com.pms.ocp.model.vo.ModelAcceptListVo;
import com.pms.ocp.model.vo.ModelAcceptVo;
import com.pms.ocp.model.vo.ModelCheckVo;
import com.pms.ocp.model.vo.ResponseVO;
import com.pms.ocp.service.MiddleGroundModelService;
import com.pms.ocp.service.ModelAcceptService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author huxiuwu
 * @version 1.0
 * @date 2022/4/7 16:13
 */
@Service
@Slf4j
public class ModelAcceptServiceImpl implements ModelAcceptService {

    @Resource
    private ModelAcceptMapper modelAcceptMapper;

    @Resource
    private MiddleGroundModelService middleGroundModelService;

    @Resource
    private ModelMapper modelMapper;

    @Resource
    private ModelCheckMapper modelCheckMapper;

    @Resource
    private ModelAuditMapper modelAuditMapper;


    /**
     * 模型验收列表
     *
     * @return pms.ocp.model.vo.ResponseVO
     * @author huxiuwu
     * @date 2022/4/7
     **/
    @Override
    public ResponseVO modelAcceptList() {
        List<ModelAcceptListVo> list = modelAcceptMapper.modelAcceptList();
        return ResponseVO.ok(list);
    }

    /**
     * 模型验收详情
     *
     * @param modelCode       模型编码
     * @param requirementCode 需求编码
     * @return pms.ocp.model.vo.ResponseVO
     * @author huxiuwu
     * @date 2022/4/8
     **/
    @Override
    public ResponseVO<ModelAcceptVo> modelAuditDetail(String modelCode, String requirementCode) {
        List<ModelAcceptVo> vos = modelAcceptMapper.modelAuditDetail(requirementCode);
        //关联模型code
        List<ModelCheckVo> relationModel = modelAcceptMapper.relationModels(modelCode);
        (vos.get(0) == null ? new ModelAcceptVo() : vos.get(0)).setRelationModels(relationModel);
        return ResponseVO.ok(vos.get(0));
    }

    /**
     * 模型信息对比
     *
     * @param requirementCode 需求编码
     * @param modelCode       模型编码
     * @param tableName       表名称
     * @return ResponseVO<java.util.List < com.pms.ocp.model.entity.ModelPropInfo>>
     * @author huxiuwu
     * @date 2022/4/11
     **/
    @Override
    public ResponseVO<List<ModelPropInfo>> modelDetailCompare(String requirementCode, String modelCode, String tableName) {
        //查询中台库数据
        ModelPropInfo modelPropInfo = middleGroundModelService.selectMiddleGroundModeByName(tableName);
        //查询自己的属性
        List<PropInfo> propList = modelAcceptMapper.selectPropList(modelCode);
        //比较并封装
        List<ModelPropInfo> result = compareData(modelPropInfo, propList, tableName);

        return ResponseVO.ok(result);
    }

    /**
     * @param model 模型数据
     * @return ResponseVO<java.util.List com.pms.ocp.model.entity.ModelPropInfo>>
     * @author huxiuwu
     * @date 2022/4/11
     **/
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseVO<List<ModelPropInfo>> modelAcceptPass(Model model) {
        //新增至模型基础表
        modelMapper.insert(model);
        //新增至模型事记表
        ModelAudit modelAudit = pageModelAudit(model);
        modelAuditMapper.insert(modelAudit);
        //更行属性信息到模型属性基础表
        String code = model.getModelCode() == null ? UUID.randomUUID().toString() : model.getModelCode();
        List<ModelPropInfo> modelPropInfoList = modelCheckMapper.selectModelOwnerProp(code);
        if (CollectionUtil.isNotEmpty(modelPropInfoList)
                && CollectionUtil.isNotEmpty(modelPropInfoList.get(0).getPropList())
                && CollectionUtil.isNotEmpty(modelPropInfoList.get(0).getPropList())) {
            List<PropInfo> propList = modelPropInfoList.get(0).getPropList();
            List<PropInfo> needPropList = new LinkedList<>();
            for (PropInfo propInfo : propList) {
                if (propInfo.getPropCode() != null && propInfo.getPropName() != null) {
                    needPropList.add(propInfo);
                }
            }
            if (CollectionUtil.isNotEmpty(needPropList)){
                modelAcceptMapper.insertPropList(needPropList);
            }

        }
        //删除模型版本表,属性版本表,需求表,审核表
        modelAcceptMapper.deleteModelVersion(model);
        modelAcceptMapper.deletePropVersion(model);
        modelAcceptMapper.deleteRequirementVersion(model);
        modelAcceptMapper.deleteAcceptVersion(model);
        return ResponseVO.ok();
    }

    private ModelAudit pageModelAudit(Model model) {
        ModelAudit modelAudit = new ModelAudit();
        modelAudit.setAuditCtime(LocalDateTime.now());
        modelAudit.setAuditType(0);
        modelAudit.setOperStatus(model.getOperStatus());
        modelAudit.setApplyUserId(model.getModelUserId());
        modelAudit.setAuditCompanyCode(model.getOwnerCompanyCode());
        modelAudit.setAuditUserName(model.getModelUserName());
        modelAudit.setModelCode(model.getModelCode());
        modelAudit.setApplyVersion(model.getModelVersion());
        modelAudit.setObjId(UUID.randomUUID().toString());
        modelAudit.setPreModelVersion(model.getModelVersion());
        return modelAudit;
    }

    /**
     * 比较是否相似
     *
     * @param modelPropInfo
     * @param propList
     * @param tableName
     * @return List<com.pms.ocp.model.entity.ModelPropInfo>
     * @author huxiuwu
     * @date 2022/4/11
     **/
    private List<ModelPropInfo> compareData(ModelPropInfo modelPropInfo, List<PropInfo> propList, String tableName) {
        List<ModelPropInfo> list = new LinkedList<>();
        ModelPropInfo owner = new ModelPropInfo();
        List<PropInfo> middleGroundList = modelPropInfo.getPropList();
        owner.setModelTable(tableName);
        if (CollectionUtil.isEmpty(middleGroundList) || CollectionUtil.isEmpty(propList)) {
            owner.setPropList(propList);
            list.add(owner);
            list.add(modelPropInfo);
            return list;
        }
        List<PropInfo> ownerList = propList.stream().map(m -> {
            middleGroundList.stream().filter(m2 -> Objects.equals(m.getPropName(), m2.getPropName())).forEach(m2 ->
                    m.setSame(true)
            );
            return m;
        }).collect(Collectors.toList());
        owner.setPropList(ownerList);
        list.add(owner);
        list.add(modelPropInfo);
        return list;
    }

}
